# coding=utf-8
"""Drawing 4 shapes with different transformations"""

import glfw
from OpenGL.GL import *
import numpy as np
import grafica.basic_shapes as bs
import grafica.easy_shaders as es
import grafica.transformations as tr
import grafica.performance_monitor as pm

__author__ = "Daniel Calderon"
__license__ = "MIT"


# We will use 32 bits data, so an integer has 4 bytes
# 1 byte = 8 bits
SIZE_IN_BYTES = 4


if __name__ == "__main__":

    # Initialize glfw
    if not glfw.init():
        glfw.set_window_should_close(window, True)

    # Creating a glfw window
    width = 600
    height = 600
    title = "Transformaciones en 2D"
    window = glfw.create_window(width, height, title, None, None)

    if not window:
        glfw.terminate()
        glfw.set_window_should_close(window, True)

    glfw.make_context_current(window)

    # Creating our shader program and telling OpenGL to use it
    pipeline = es.SimpleTransformShaderProgram()
    glUseProgram(pipeline.shaderProgram)

    # Setting up the clear screen color
    glClearColor(0.15, 0.15, 0.15, 1.0)

    # Creating shapes on GPU memory
    shapeTriangle = bs.createRainbowTriangle()
    gpuTriangle = es.GPUShape().initBuffers()
    pipeline.setupVAO(gpuTriangle)
    gpuTriangle.fillBuffers(shapeTriangle.vertices, shapeTriangle.indices, GL_STATIC_DRAW)



    perfMonitor = pm.PerformanceMonitor(glfw.get_time(), 0.5)

    # glfw will swap buffers as soon as possible
    glfw.swap_interval(0)

    # Application loop
    while not glfw.window_should_close(window):

        # Measuring performance
        perfMonitor.update(glfw.get_time())
        glfw.set_window_title(window, title + str(perfMonitor))

        # Using GLFW to check for input events
        glfw.poll_events()

        # Clearing the screen
        glClear(GL_COLOR_BUFFER_BIT)

        # Using the time as the theta parameter
        theta = glfw.get_time()

        triangleTransform = tr.translate(1, 1, 0)
        #triangleTransform = tr.uniformScale(0.5)
        #triangleTransform = tr.scale(2,1,1)
        #triangleTransform = tr.shearing(0, 1, 0, 0, 0, 0)
        #triangleTransform = tr.shearing(0.5 * np.cos(theta), 0, 0, 0, 0, 0)
        #triangleTransform = tr.shearing(0.3 * theta, 0, 0, 0, 0, 0)
        #triangleTransform = tr.rotationX(2)
        #triangleTransform = tr.rotationY(2 * theta)
        #triangleTransform = tr.rotationZ(2 * theta)

        #glClearColor(np.cos(7*theta), 0, 0, 1.0)
        #glClearColor(np.cos(7*theta), 0.5+ np.cos(2*theta), 0.7+np.cos(100*theta), 1.0)

        '''
        # Triangle
        triangleTransform = tr.matmul([
            tr.translate(0.9, 0, 0),
            tr.uniformScale(1)
        ])
        '''




        # Updating the transform attribute
        glUniformMatrix4fv(glGetUniformLocation(pipeline.shaderProgram, "transform"), 1, GL_TRUE, triangleTransform)



        # Drawing function
        pipeline.drawCall(gpuTriangle)

        # Once the drawing is rendered, buffers are swap so an uncomplete drawing is never seen.
        glfw.swap_buffers(window)

    # freeing GPU memory
    gpuTriangle.clear()
    
    glfw.terminate()